@@ -35,27 +35,29 @@ import logging
 import networkx
 import os
 import tempfile
-from collections.abc import Callable, Awaitable
+from collections.abc import Callable, Awaitable, Iterable
 from functools import cache
 from pathlib import Path
-from slugify import slugify
 import queue
 from typing import Any, Generator
 
 import git
 
 from . import kustomize
+from .exceptions import FluxException
 from .manifest import (
     CRD_KIND,
     CLUSTER_KUSTOMIZE_DOMAIN,
     KUSTOMIZE_DOMAIN,
     Cluster,
+    ClusterPolicy,
     HelmRelease,
     HelmRepository,
     Kustomization,
     Manifest,
     SECRET_KIND,
 )
+from .exceptions import InputException
 
 __all__ = [
     "build_manifest",
@@ -70,6 +72,7 @@ CLUSTER_KUSTOMIZE_KIND = "Kustomization"
 KUSTOMIZE_KIND = "Kustomization"
 HELM_REPO_KIND = "HelmRepository"
 HELM_RELEASE_KIND = "HelmRelease"
+CLUSTER_POLICY_KIND = "ClusterPolicy"
 GIT_REPO_KIND = "GitRepository"
 DEFAULT_NAMESPACE = "flux-system"
 
@@ -77,9 +80,12 @@ DEFAULT_NAMESPACE = "flux-system"
 @cache
 def git_repo(path: Path | None = None) -> git.repo.Repo:
     """Return the local git repo path."""
-    if path is None:
-        return git.repo.Repo(os.getcwd(), search_parent_directories=True)
-    return git.repo.Repo(str(path), search_parent_directories=True)
+    try:
+        if path is None:
+            return git.repo.Repo(os.getcwd(), search_parent_directories=True)
+        return git.repo.Repo(str(path), search_parent_directories=True)
+    except git.GitError as err:
+        raise InputException(f"Unable to find input path {path}: {err}") from err
 
 
 @cache
@@ -147,7 +153,7 @@ class ResourceVisitor:
         [
             Path,
             Path,
-            Kustomization | HelmRelease | HelmRepository,
+            Kustomization | HelmRelease | HelmRepository | ClusterPolicy,
             kustomize.Kustomize | None,
         ],
         Awaitable[None],
@@ -188,11 +194,13 @@ class MetadataSelector:
     @property
     def predicate(
         self,
-    ) -> Callable[[Kustomization | HelmRelease | Cluster | HelmRepository], bool]:
+    ) -> Callable[
+        [Kustomization | HelmRelease | Cluster | HelmRepository | ClusterPolicy], bool
+    ]:
         """A predicate that selects Kustomization objects."""
 
         def predicate(
-            obj: Kustomization | HelmRelease | Cluster | HelmRepository,
+            obj: Kustomization | HelmRelease | Cluster | HelmRepository | ClusterPolicy,
         ) -> bool:
             if not self.enabled:
                 return False
@@ -239,6 +247,9 @@ class ResourceSelector:
     helm_release: MetadataSelector = field(default_factory=MetadataSelector)
     """HelmRelease objects to return."""
 
+    cluster_policy: MetadataSelector = field(default_factory=MetadataSelector)
+    """ClusterPolicy objects to return."""
+
 
 async def get_flux_kustomizations(
     root: Path, relative_path: Path
@@ -279,7 +290,18 @@ async def kustomization_traversal(path_selector: PathSelector) -> list[Kustomiza
     while not path_queue.empty():
         path = path_queue.get()
         _LOGGER.debug("Visiting path (%s) %s", root, path)
-        docs = await get_flux_kustomizations(root, path)
+        try:
+            docs = await get_flux_kustomizations(root, path)
+        except FluxException as err:
+            if visited:
+                raise FluxException(
+                    f"Error building Fluxtomization in '{root}' path '{path}': {err}"
+                    f"Is a Kustomization pointing to a path that does not exist?"
+                )
+            raise FluxException(
+                f"Error building Fluxtomization in '{root}' path '{path}': {err}"
+                f"Try specifying another path within the git repo?"
+            )
 
         # Source path is relative to the search path. Update to have the
         # full prefix relative to the root.
@@ -323,7 +345,9 @@ def make_clusters(kustomizations: list[Kustomization]) -> list[Cluster]:
     parent_paths = set([Path(ks.path) for ks in kustomizations])
     for ks in kustomizations:
         if not ks.source_path:
-            raise ValueError("Kustomization did not have source path; Old kustomize?")
+            raise InputException(
+                "Kustomization did not have source path; Old kustomize?"
+            )
         path = Path(ks.path)
         source = Path(ks.source_path)
         graph.add_node(path, ks=ks)
@@ -392,18 +416,19 @@ async def build_kustomization(
     kustomization: Kustomization,
     cluster_path: Path,
     root: Path,
-    stash_prefix: Path,
     kustomization_selector: MetadataSelector,
     helm_release_selector: MetadataSelector,
     helm_repo_selector: MetadataSelector,
-) -> tuple[list[HelmRepository], list[HelmRelease]]:
+    cluster_policy_selector: MetadataSelector,
+) -> tuple[Iterable[HelmRepository], Iterable[HelmRelease], Iterable[ClusterPolicy]]:
     """Build helm objects for the Kustomization."""
     if (
-        not kustomization_selector.visitor
+        not kustomization_selector.enabled
         and not helm_release_selector.enabled
         and not helm_repo_selector.enabled
+        and not cluster_policy_selector.enabled
     ):
-        return ([], [])
+        return ([], [], [])
 
     cmd = kustomize.build(root / kustomization.path)
     skips = []
@@ -412,17 +437,15 @@ async def build_kustomization(
     if kustomization_selector.skip_secrets:
         skips.append(SECRET_KIND)
     cmd = cmd.skip_resources(skips)
-    cmd = await cmd.stash(
-        Path(
-            "-".join(
-                [
-                    str(stash_prefix),
-                    slugify(kustomization.path),
-                    slugify(kustomization.name),
-                ]
-            )
-        )
-    )
+    try:
+        cmd = await cmd.stash()
+    except FluxException as err:
+        raise FluxException(
+            f"Error while building Kustomization "
+            f"'{kustomization.namespace}/{kustomization.name}' "
+            f"(path={kustomization.source_path}): {err}"
+        ) from err
+
     if kustomization_selector.visitor:
         if kustomization_selector.visitor:
             await kustomization_selector.visitor.func(
@@ -432,25 +455,40 @@ async def build_kustomization(
                 cmd,
             )
 
-    if not helm_release_selector.enabled and not helm_repo_selector.enabled:
-        return ([], [])
+    if (
+        not helm_release_selector.enabled
+        and not helm_repo_selector.enabled
+        and not cluster_policy_selector.enabled
+    ):
+        return ([], [], [])
 
-    (repo_docs, release_docs) = await asyncio.gather(
-        cmd.grep(f"kind=^{HELM_REPO_KIND}$").objects(),
-        cmd.grep(f"kind=^{HELM_RELEASE_KIND}$").objects(),
-    )
+    docs = await cmd.grep(
+        f"kind=^({HELM_REPO_KIND}|{HELM_RELEASE_KIND}|{CLUSTER_POLICY_KIND})$"
+    ).objects()
     return (
-        list(
-            filter(
-                helm_repo_selector.predicate,
-                [HelmRepository.parse_doc(doc) for doc in repo_docs],
-            )
+        filter(
+            helm_repo_selector.predicate,
+            [
+                HelmRepository.parse_doc(doc)
+                for doc in docs
+                if doc.get("kind") == HELM_REPO_KIND
+            ],
         ),
-        list(
-            filter(
-                helm_release_selector.predicate,
-                [HelmRelease.parse_doc(doc) for doc in release_docs],
-            )
+        filter(
+            helm_release_selector.predicate,
+            [
+                HelmRelease.parse_doc(doc)
+                for doc in docs
+                if doc.get("kind") == HELM_RELEASE_KIND
+            ],
+        ),
+        filter(
+            cluster_policy_selector.predicate,
+            [
+                ClusterPolicy.parse_doc(doc)
+                for doc in docs
+                if doc.get("kind") == CLUSTER_POLICY_KIND
+            ],
         ),
     )
 
@@ -489,62 +527,67 @@ async def build_manifest(
             ]
             clusters.append(cluster)
 
-    async def update_kustomization(cluster: Cluster, stash_dir: str) -> None:
-        cluster_stash = Path(stash_dir) / slugify(cluster.path)
-        helm_tasks = []
+    async def update_kustomization(cluster: Cluster) -> None:
+        build_tasks = []
         for kustomization in cluster.kustomizations:
-            _LOGGER.debug(
-                "Processing kustomization: %s (stash=%s)",
-                kustomization.path,
-                cluster_stash,
-            )
-            helm_tasks.append(
+            _LOGGER.debug("Processing kustomization: %s", kustomization.path)
+            build_tasks.append(
                 build_kustomization(
                     kustomization,
                     Path(cluster.path),
                     selector.path.root,
-                    cluster_stash,
                     selector.kustomization,
                     selector.helm_release,
                     selector.helm_repo,
+                    selector.cluster_policy,
                 )
             )
-        results = list(await asyncio.gather(*helm_tasks))
-        for kustomization, (helm_repos, helm_releases) in zip(
+        results = list(await asyncio.gather(*build_tasks))
+        for kustomization, (helm_repos, helm_releases, cluster_policies) in zip(
             cluster.kustomizations,
             results,
         ):
-            kustomization.helm_repos = helm_repos
-            kustomization.helm_releases = helm_releases
+            kustomization.helm_repos = list(helm_repos)
+            kustomization.helm_releases = list(helm_releases)
+            kustomization.cluster_policies = list(cluster_policies)
 
-    with tempfile.TemporaryDirectory() as stash_dir:
-        kustomization_tasks = []
-        # Expand and visit Kustomizations
-        for cluster in clusters:
-            kustomization_tasks.append(update_kustomization(cluster, stash_dir))
-        await asyncio.gather(*kustomization_tasks)
+    kustomization_tasks = []
+    # Expand and visit Kustomizations
+    for cluster in clusters:
+        kustomization_tasks.append(update_kustomization(cluster))
+    await asyncio.gather(*kustomization_tasks)
 
-        # Visit Helm resources
-        for cluster in clusters:
-            if selector.helm_repo.visitor:
-                for kustomization in cluster.kustomizations:
-                    for helm_repo in kustomization.helm_repos:
-                        await selector.helm_repo.visitor.func(
-                            Path(cluster.path),
-                            Path(kustomization.path),
-                            helm_repo,
-                            None,
-                        )
+    # Visit Helm resources
+    for cluster in clusters:
+        if selector.helm_repo.visitor:
+            for kustomization in cluster.kustomizations:
+                for helm_repo in kustomization.helm_repos:
+                    await selector.helm_repo.visitor.func(
+                        Path(cluster.path),
+                        Path(kustomization.path),
+                        helm_repo,
+                        None,
+                    )
 
-            if selector.helm_release.visitor:
-                for kustomization in cluster.kustomizations:
-                    for helm_release in kustomization.helm_releases:
-                        await selector.helm_release.visitor.func(
-                            Path(cluster.path),
-                            Path(kustomization.path),
-                            helm_release,
-                            None,
-                        )
+        if selector.helm_release.visitor:
+            for kustomization in cluster.kustomizations:
+                for helm_release in kustomization.helm_releases:
+                    await selector.helm_release.visitor.func(
+                        Path(cluster.path),
+                        Path(kustomization.path),
+                        helm_release,
+                        None,
+                    )
+
+        if selector.cluster_policy.visitor:
+            for kustomization in cluster.kustomizations:
+                for cluster_policy in kustomization.cluster_policies:
+                    await selector.cluster_policy.visitor.func(
+                        Path(cluster.path),
+                        Path(kustomization.path),
+                        cluster_policy,
+                        None,
+                    )
 
     return Manifest(clusters=clusters)
 
