@@ -45,6 +45,7 @@ import yaml
 from . import command
 from .kustomize import Kustomize
 from .manifest import HelmRelease, HelmRepository, CRD_KIND, SECRET_KIND
+from .exceptions import HelmException
 
 __all__ = [
     "Helm",
@@ -115,7 +116,11 @@ class Helm:
         content = yaml.dump(RepositoryConfig(self._repos).config, sort_keys=False)
         async with aiofiles.open(str(self._repo_config_file), mode="w") as config_file:
             await config_file.write(content)
-        await command.run(command.Command([HELM_BIN, "repo", "update"] + self._flags))
+        await command.run(
+            command.Command(
+                [HELM_BIN, "repo", "update"] + self._flags, exc=HelmException
+            )
+        )
 
     async def template(
         self,
@@ -124,6 +129,8 @@ class Helm:
         skip_crds: bool = True,
         skip_tests: bool = True,
         skip_secrets: bool = False,
+        kube_version: str | None = None,
+        api_versions: str | None = None,
     ) -> Kustomize:
         """Return command line arguments to template the specified chart.
 
@@ -150,6 +157,10 @@ class Helm:
                     release.chart.version,
                 ]
             )
+        if kube_version:
+            args.extend(["--kube-version", kube_version])
+        if api_versions:
+            args.extend(["--api-versions", api_versions])
         if release.values and not values:
             values = release.values
         if values:
@@ -157,7 +168,7 @@ class Helm:
             async with aiofiles.open(values_path, mode="w") as values_file:
                 await values_file.write(yaml.dump(values, sort_keys=False))
             args.extend(["--values", str(values_path)])
-        cmd = Kustomize([command.Command(args + self._flags)])
+        cmd = Kustomize([command.Command(args + self._flags, exc=HelmException)])
         skips = []
         if skip_crds:
             skips.append(CRD_KIND)
